from nullptr.store import Store
from nullptr.models.ship import Ship
from nullptr.missions import create_mission, get_mission_class
from nullptr.models.waypoint import Waypoint
from random import choice, randrange
from time import sleep, time
from threading import Thread
from nullptr.atlas_builder import  AtlasBuilder
from nullptr.general import General
from nullptr.util import *
from nullptr.roles import assign_mission

class CentralCommandError(AppError):
  pass
  
class Captain:
  def __init__(self, context):
    self.missions = {}
    self.stopping = False
    self.store = context.store
    self.c = context
    self.general = context.general
    self.api = context.api
    self.general = context.general
    self.atlas_builder = AtlasBuilder(self.store, self.api)
    
  def setup(self):
    self.update_missions()
  
  def get_ready_missions(self):
    result = []
    prio = 1
    for ship, mission in self.missions.items():
      p = mission.is_ready()
      if p == prio:
        result.append(ship)
      elif p > prio:
        prio = p
        result = [ship]
    return result
  
  def single_step(self, ship):
    if ship not in self.missions:
      print('ship has no mission')
    mission = self.missions[ship]
    mission.step()
    
  def tick(self):
    self.general.tick()
    self.update_missions()
    missions = self.get_ready_missions()
    if len(missions) == 0: return False
    ship = choice(missions)
    mission = self.missions[ship]
    mission.step()
    return True
    
  def run_interactive(self):
    print('auto mode. hit enter to stop')
    t = Thread(target=self.wait_for_stop)
    t.daemon = True
    t.start()
    self.run()
    print('manual mode')
  
  def wait_for_stop(self):
    try:
      input()
    except EOFError:
      pass
    self.stopping = True
    print('stopping...')
  
  def run(self):
    while not self.stopping:
      # any new orders?
      self.c.general.tick()
      did_step = True
      request_counter = self.api.requests_sent
      start = time()
      while request_counter == self.api.requests_sent and did_step:
        did_step = self.tick()
      if request_counter == self.api.requests_sent:
        self.atlas_builder.do_work()
      else:
        pass # print('nowork')
        
      self.store.flush()
      dur = time() - start
      # print(f'step {dur:.03}')
      zs = 0.5 - dur
      if zs > 0:
        sleep(zs)
    self.stopping = False
      
  def stop(self):
    self.stopping = True
 
  def set_mission_param(self, ship, nm, val):
    if ship not in self.missions:
      print('set a mission for this ship first')
      return
    mission = self.missions[ship]
    params = mission.params()
    if not nm in params:
      print(f'{nm} is not a valid param')
      return
    param = params[nm]
    try:
      parsed_val = param.parse(val, self.store)
    except ValueError as e:
      raise MissionError(e)
      return
    ship.set_mission_state(nm, parsed_val)
  
  def smipa(self,s,n,v):    
    self.set_mission_param(s,n,v)
    
  def update_missions(self):
    for s in self.store.all(Ship):
      if s.mission_status == 'done':
        s.mission = None
      if s.mission is None:
        if s in self.missions:
          self.stop_mission(s)
      if s.mission is None:
        assign_mission(self.c, s)
      if s.mission is not None and s not in self.missions:
        self.start_mission(s)
      if s in self.missions:
        m = self.missions[s]
    
  def init_mission(self, s, mtyp):
    if mtyp == 'none':
      s.mission_state = {}
      s.mission_status = None
      s.mission = None
      return
    try:
      mclass = get_mission_class(mtyp)
    except ValueError:
      raise CentralCommandError('no such mission')
    s.mission = mtyp
    s.mission_status = 'init'
    s.mission_state = {k: v.default for k,v in mclass.params().items()}
    self.start_mission(s)
  
  def restart_mission(self, s, status='init'):
    if s not in self.missions:
      raise CentralCommandError("no mission assigned")
    s.mission_status = status
    
  def start_mission(self, s):
    mtype = s.mission
    m = create_mission(mtype, s, self.c)
    self.missions[s] = m
    m.status(s.mission_status)
    return m
    
  def stop_mission(self, s):
    if s in self.missions:
      del self.missions[s]

  